;;; This file is part of guile-with-graphics, a graphic library for Guile

;;; Copyright (C) 2009  Luca Saiu

;;; This program is free software: you can redistribute it and/or modify
;;; it under the terms of the GNU General Public License as published by
;;; the Free Software Foundation, either version 3 of the License, or
;;; (at your option) any later version.

;;; This program is distributed in the hope that it will be useful,
;;; but WITHOUT ANY WARRANTY; without even the implied warranty of
;;; MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
;;; GNU General Public License for more details.

;;; You should have received a copy of the GNU General Public License
;;; along with this program.  If not, see <http://www.gnu.org/licenses/>.


(use-modules (ice-9 format))

(load "../utility.scm")
(load "../network.scm")
(load "big-project-demo-rules.scm")

;;; Handle the command line:
(define the-command-line (command-line))
(unless (= (length the-command-line) 4)
  (format #t "ERROR: There must be exactly three command-line parameters:\n")
  (format #t "       server name, port, nickname\n")
  (exit -1))
(define server-name (cadr the-command-line))
(define server-port (string->number (caddr the-command-line)))
(define nick (cadddr the-command-line))
(define client-to-server-port (make-client-to-server-port server-name server-port))

(define (send-message message)
  (format client-to-server-port "~s\n" message))

(define (receive-message)
  (read client-to-server-port))

(define players '())
(define pills '())
(define my-player #f) ;; #f means that I don't have information from the server yet

(define (update-state! message-from-server)
  (cond ((eq? (car message-from-server) 'updated-state)
         (set! players (cadr message-from-server))
         (set! pills (caddr message-from-server))
         ;; Notice that we are assuming that we are the only player with our nick
         (set! my-player (car (filter (lambda (player) (equal? (car player) nick))
                                      players))))
        (else
         ;; Do nothing
         #t)))

(define (nearest-pill x y)
  (define (nearest-pill pills best-distance-till-now best-pill-till-now)
    (if (null? pills)
        best-pill-till-now
        (let* ((first-pill (car pills))
               (first-pill-distance (distance-from-pill x y first-pill)))
          (if (< first-pill-distance best-distance-till-now)
              (nearest-pill (cdr pills) first-pill-distance first-pill)
              (nearest-pill (cdr pills) best-distance-till-now best-pill-till-now)))))
  (nearest-pill (cdr pills) (distance-from-pill x y (car pills)) (car pills)))

(define (decide-and-send-commands)
  (let* ((interesting-part (cddr my-player))
         (x (car interesting-part))
         (y (cadr interesting-part))
         (angle (caddr interesting-part))
         (nearest-pill (nearest-pill x y))
         (angle-to-target (angle-to-target x y angle (car nearest-pill) (cdr nearest-pill))))
    (if (> (abs angle-to-target) 0.3)
        (begin
          (send-message `(set-rotation ,(if (> angle-to-target 0) 'left 'right)))
          (send-message '(set-translation none))
        (begin
          (send-message '(set-rotation none))
          (send-message '(set-translation forward)))))))    

;;; This is the first message the server must receive:
(send-message nick)

(define mutex (make-mutex))

(make-thread (lambda ()
               (while #t
                 (with-mutex mutex
                   (update-state! (receive-message))))))

(define (on-tick seconds-since-last-tick)
  (with-mutex mutex
    (when my-player
      (decide-and-send-commands))))

(call-with-maximum-frequency 100 on-tick)
