;;; This file is part of guile-with-graphics, a graphic library for Guile

;;; Copyright (C) 2009  Luca Saiu

;;; This program is free software: you can redistribute it and/or modify
;;; it under the terms of the GNU General Public License as published by
;;; the Free Software Foundation, either version 3 of the License, or
;;; (at your option) any later version.

;;; This program is distributed in the hope that it will be useful,
;;; but WITHOUT ANY WARRANTY; without even the implied warranty of
;;; MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
;;; GNU General Public License for more details.

;;; You should have received a copy of the GNU General Public License
;;; along with this program.  If not, see <http://www.gnu.org/licenses/>.


(use-modules (ice-9 format))

(load "../utility.scm")
(load "../network.scm")
(load "big-project-demo-rules.scm")

;;; Handle the command line:
(define the-command-line (command-line))
(unless (= (length the-command-line) 4)
  (format #t "ERROR: There must be exactly three command-line parameters:\n")
  (format #t "       server name, port, nickname\n")
  (exit -1))
(define server-name (cadr the-command-line))
(define server-port (string->number (caddr the-command-line)))
(define nick (cadddr the-command-line))
(define client-to-server-port (make-client-to-server-port server-name server-port))

(initialize-graphics (string-append "Demo client (connected as " nick " to " server-name " on port " (number->string server-port) ")")
                     field-width field-height)

(load "../color.scm") ;; this must be loaded *after* graphics is initialized
(load "../graphics.scm")
(load "../interactive.scm")
(load "big-project-demo-graphics.scm")

(define (send-message message)
  (format client-to-server-port "~s\n" message))

(define (receive-message)
  (read client-to-server-port))

(define (handle-event event)
  (cond ((quit-event? event)
         (exit 0))
        ((equal? event '(key-down left))
         (send-message '(set-rotation left)))
        ((equal? event '(key-down right))
         (send-message '(set-rotation right)))
        ((or (equal? event '(key-up left))
             (equal? event '(key-up right)))
         (send-message '(set-rotation none)))
        ((equal? event '(key-down up))
         (send-message '(set-translation forward)))
        ((equal? event '(key-down down))
         (send-message '(set-translation backward)))
        ((or (equal? event '(key-up up))
             (equal? event '(key-up down)))
         (send-message '(set-translation none)))
        ((equal? event '(key-down #\d))
         (rotate-detail-level!))))

(define players '())
(define pills '())

(define (draw-player-from-client-state name score x y angle red green blue)
  (draw-player x
               y
               player-radius
               angle
               (string-append name " (" (number->string score) ")")
               (make-color red green blue)))

(define (draw-current-state)
  (iterate (lambda (player-from-client-state)
             (apply draw-player-from-client-state
                    player-from-client-state))
           players)
  (iterate draw-pill pills))

(define (update-state! message-from-server)
  (cond ((eq? (car message-from-server) 'updated-state)
         (set! players (cadr message-from-server))
         (set! pills (caddr message-from-server)))
        ((eq? (car message-from-server) 'broadcast)
         (iterate console-write-line! (reverse (cdr message-from-server))))
        (else
         (console-write-line! (string-append "unknown message: "
                                             (s-expression->string message-from-server))))))

;;; This is the first message the server must receive:
(send-message nick)

(define mutex (make-mutex))

(make-thread (lambda ()
               (while #t
                 (with-mutex mutex
                   (update-state! (receive-message))))))

(define (on-tick seconds-since-last-tick)
  (with-mutex mutex
    (handle-all-enqueued-events handle-event)
    (clear black)
    (draw-current-state)
    (console-draw)
    (refresh)))

(call-with-maximum-frequency 40 on-tick)
