;;; This file is part of guile-with-graphics, a graphic library for Guile

;;; Copyright (C) 2009  Luca Saiu

;;; This program is free software: you can redistribute it and/or modify
;;; it under the terms of the GNU General Public License as published by
;;; the Free Software Foundation, either version 3 of the License, or
;;; (at your option) any later version.

;;; This program is distributed in the hope that it will be useful,
;;; but WITHOUT ANY WARRANTY; without even the implied warranty of
;;; MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
;;; GNU General Public License for more details.

;;; You should have received a copy of the GNU General Public License
;;; along with this program.  If not, see <http://www.gnu.org/licenses/>.


(load "../network.scm")
(load "../utility.scm")
(randomize-random-state!)
(load "big-project-demo-rules.scm")
(load "big-project-demo-state.scm")

(use-modules (ice-9 threads))
(use-modules (ice-9 format))
(use-modules (ice-9 rdelim))
(use-modules (srfi srfi-9))

;;; Don't be killed by a SIGPIPE occurring when a client closes a connection:
(sigaction SIGPIPE SIG_IGN)

(define server-port 12345)
(define server-tick-length (round (/ 1000 30)))

(define mutex (make-mutex))

(define (check-that-threads-work)
  (let ((useless-thread (make-thread (lambda (x) x) 42)))
    ;; If we're still alive here then Guile was compiled with thread support.
    #t))

(define (serve-incoming-connections)
  (let ((server-socket (make-server-socket server-port)))
    (while #t
      (let ((port-to-client (accept-connection-from-client server-socket)))
        (serve-new-connection-in-a-new-thread port-to-client)))))

(define (serve-new-connection-in-a-new-thread port-to-client)
  (make-thread serve-new-connection-in-the-current-thread
               port-to-client))

(define (serve-new-connection-in-the-current-thread port-to-client)
  (let* ((name (read port-to-client))
               (player (make-player name port-to-client)))
    (with-mutex mutex
      (add-player! player)
      (add-message-to-broadcast! (string-append "Welcome " name)))
    (catch #t
      (lambda ()
        (while #t
          (let ((message-from-the-client (read port-to-client)))
            (with-mutex mutex
              (serve-command player port-to-client message-from-the-client)))))
      (lambda (key . arguments)
        (format #t "Failed (~s: ~s).\n" key arguments)))
    (with-mutex mutex
      (remove-player! player)
      (add-message-to-broadcast! (string-append "Goodbye " name)))
    (close port-to-client)))

(define (serve-command player port-to-client command)
  (cond ((eof-object? command)
         (throw 'the-port-was-closed))
        ((equal? command '(set-rotation left))
         (player-set-rotation-state! player 'left))
        ((equal? command '(set-rotation right))
         (player-set-rotation-state! player 'right))
        ((equal? command '(set-rotation none))
         (player-set-rotation-state! player 'idle))
        ((equal? command '(set-translation forward))
         (player-set-translation-state! player 'forward))
        ((equal? command '(set-translation backward))
         (player-set-translation-state! player 'backward))
        ((equal? command '(set-translation none))
         (player-set-translation-state! player 'idle))))

(define broadcast-messages '())
(define (add-message-to-broadcast! string)
  (set! broadcast-messages (cons string broadcast-messages)))

(define (work delta-t)
  (with-mutex mutex
    (let ((update-message (update-state! delta-t)))
      (iterate (lambda (player)
                 (ignore-failures
                   (write update-message (player-port player))))
               players)
      (unless (null? broadcast-messages)
        (iterate (lambda (player)
                   (ignore-failures
                     (write `(broadcast ,@broadcast-messages)
                           (player-port player))))
                 players))
      (set! broadcast-messages '()))))

(check-that-threads-work)
(make-thread serve-incoming-connections)
(call-with-maximum-frequency 40 work)
