;;; This file is part of guile-with-graphics, a graphic library for Guile

;;; Copyright (C) 2009  Luca Saiu

;;; This program is free software: you can redistribute it and/or modify
;;; it under the terms of the GNU General Public License as published by
;;; the Free Software Foundation, either version 3 of the License, or
;;; (at your option) any later version.

;;; This program is distributed in the hope that it will be useful,
;;; but WITHOUT ANY WARRANTY; without even the implied warranty of
;;; MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
;;; GNU General Public License for more details.

;;; You should have received a copy of the GNU General Public License
;;; along with this program.  If not, see <http://www.gnu.org/licenses/>.


(load "big-project-demo-rules.scm")

(use-modules (ice-9 format))

(initialize-graphics "Interactive game standalone demo" field-width field-height)

(load "../color.scm") ;; this must be loaded *after* graphics is initialized
(load "../utility.scm")
(load "../graphics.scm")
(load "../interactive.scm")
(load "big-project-demo-graphics.scm")
(load "big-project-demo-state.scm")

(randomize-random-state!)

(define p (make-player "luca" #f))
(add-player! p)

(define bots-no 0)
(define bots
  (map (lambda (index)
         (make-player (string-append "bot" (number->string (+ index 1))) #f))
       (iota bots-no)))
(iterate add-player! bots)

(define (handle-event event)
  (cond ((quit-event? event)
         (exit 0))
        ((equal? event '(key-down #\b))
         (let ((new-bot (make-player (string-append "bot" (number->string (+ (length bots) 1))) #f)))
           (set! bots (cons new-bot bots))
           (add-player! new-bot)))
        ((equal? event '(key-down left))
         (player-set-rotation-state! p 'left))
        ((equal? event '(key-down right))
         (player-set-rotation-state! p 'right))
        ((or (equal? event '(key-up left))
             (equal? event '(key-up right)))
         (player-set-rotation-state! p 'idle))
        ((equal? event '(key-down up))
         (player-set-translation-state! p 'forward))
        ((equal? event '(key-down down))
         (player-set-translation-state! p 'backward))
        ((equal? event '(key-down #\d))
         (rotate-detail-level!))
        ((or (equal? event '(key-up up))
             (equal? event '(key-up down)))
         (player-set-translation-state! p 'idle))))

(define pills
  (map (lambda (whatever)
         (make-pill))
       (iota pills-no)))

(define (nearest-pill x y)
  (define (nearest-pill pills best-distance-till-now best-pill-till-now)
    (if (null? pills)
        best-pill-till-now
        (let* ((first-pill (car pills))
               (first-pill-distance (distance-from-pill x y first-pill)))
          (if (< first-pill-distance best-distance-till-now)
              (nearest-pill (cdr pills) first-pill-distance first-pill)
              (nearest-pill (cdr pills) best-distance-till-now best-pill-till-now)))))
  (nearest-pill (cdr pills) (distance-from-pill x y (car pills)) (car pills)))

(define (update-bot-state! bot)
  (let* ((x (player-x bot))
         (y (player-y bot))
         (angle (player-angle bot))
         (nearest-pill (nearest-pill x y))
         (angle-to-target (angle-to-target x y angle (car nearest-pill) (cdr nearest-pill))))
    (if (> (abs angle-to-target) 0.3)
        (begin
          (player-set-rotation-state! bot (if (> angle-to-target 0) 'left 'right))
          (player-set-translation-state! bot 'idle))
        (begin
          (player-set-rotation-state! bot 'idle)
          (player-set-translation-state! bot 'forward)))))

(define (work delta-t)
  (handle-all-enqueued-events handle-event)
  (update-state! delta-t)
  (iterate update-bot-state! bots)
  (clear black)
  (iterate (lambda (p)
             (draw-player (player-x p)
                          (player-y p)
                          player-radius
                          (player-angle p)
                          (string-append (player-name p) " (" (number->string (player-score p)) ")")
                          (make-color (player-red p)
                                      (player-green p)
                                      (player-blue p))))
           players)
  (iterate draw-pill pills)
  (console-update-state!)
  (console-draw)
  (refresh))

(call-with-maximum-frequency 70 work)
